#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/signal.h>
#include <avr/pgmspace.h>
#include <avr/wdt.h>
#include "uart.h"
#include "global.h"
#include "isa.h"

void init_isa(void)
{
	DDRE=7;
	DDRA=0;
	DDRC=255;
	sbi (PORTC, SBHE);
	delay (1000);	
	sbi (PORTC, Reset);
	delay (10000);
	cbi (PORTC, Reset);
}

void set_adress(dw adresse)
{
	DDRA=255;
	PORTA=adresse.byte[2];
	PORTE=A2_WRITE;
	sbi (PORTC, Enable);
	asm ("nop");
	asm ("nop");
	cbi (PORTC, Enable);
	asm ("nop");
	PORTA=adresse.byte[1];
	PORTE=A1_WRITE;
	sbi (PORTC, Enable);
	asm ("nop");
	asm ("nop");
	cbi (PORTC, Enable);
	asm ("nop");
	PORTA=adresse.byte[0];
	PORTE=A0_WRITE;
	sbi (PORTC, Enable);
	asm ("nop");
	asm ("nop");
	cbi (PORTC, Enable);
	asm ("nop");
	asm ("nop");
	asm ("nop");
	asm ("nop");
	sbi (PORTC, ALE);
	asm ("nop");
	asm ("nop");
	asm ("nop");
	asm ("nop");
	cbi (PORTC, ALE);
}

void io_write(unsigned char byte)
{
	DDRA=255;
	PORTA=byte;
	PORTE=IO_WRITE;
	asm ("nop");
	asm ("nop");
	asm ("nop");
	asm ("nop");
	sbi (PORTC, Enable);
	asm ("nop");
	asm ("nop");
	asm ("nop");
	asm ("nop");
	asm ("nop");
	asm ("nop");
	asm ("nop");
	asm ("nop");
	asm ("nop");
	asm ("nop");
	cbi (PORTC, Enable);
}

void mem_write(unsigned char byte)
{
	DDRA=255;
	PORTA=byte;
	PORTE=MEM_WRITE;
	asm ("nop");
	asm ("nop");
	asm ("nop");
	asm ("nop");
	sbi (PORTC, Enable);
	asm ("nop");
	asm ("nop");
	asm ("nop");
	asm ("nop");
	asm ("nop");
	asm ("nop");
	asm ("nop");
	asm ("nop");
	asm ("nop");
	asm ("nop");
	cbi (PORTC, Enable);
}

unsigned char io_read(void)
{unsigned char byte;
	DDRA=0;
	PORTA=255;
	PORTE=IO_READ;
	asm ("nop");
	sbi (PORTC, Enable);
	asm ("nop");
	asm ("nop");
	asm ("nop");
	asm ("nop");
	asm ("nop");
	asm ("nop");
	asm ("nop");
	asm ("nop");
	asm ("nop");
	asm ("nop");
	byte=PINA;
	cbi (PORTC, Enable);
	return byte;
}

unsigned char mem_read(void)
{unsigned char byte;
	DDRA=0;
	PORTA=255;
	PORTE=MEM_READ;
	asm ("nop");
	sbi (PORTC, Enable);
	asm ("nop");
	asm ("nop");
	asm ("nop");
	asm ("nop");
	asm ("nop");
	asm ("nop");
	asm ("nop");
	asm ("nop");
	asm ("nop");
	asm ("nop");
	byte=PINA;
	cbi (PORTC, Enable);
	return byte;
}

void io_write16(unsigned int byte)
{
	DDRA=255;
	PORTA=(char)(byte/256);
	PORTE=HIGH_BYTE;
	sbi (PORTC, Enable);
	asm ("nop");
	asm ("nop");
	cbi (PORTC, Enable);
	asm ("nop");
	PORTA=(char)(byte&255);
	cbi (PORTC, SBHE);
	PORTE=IO_WRITE;
	asm ("nop");
	asm ("nop");
	asm ("nop");
	asm ("nop");
	sbi (PORTC, Enable);
	asm ("nop");
	asm ("nop");
	asm ("nop");
	asm ("nop");
	asm ("nop");
	asm ("nop");
	asm ("nop");
	asm ("nop");
	asm ("nop");
	asm ("nop");
	cbi (PORTC, Enable);
	asm ("nop");
	asm ("nop");
	sbi (PORTC, SBHE);
}

void mem_write16(unsigned int byte)
{
	DDRA=255;
	PORTA=(char)(byte/256);
	PORTE=HIGH_BYTE;
	sbi (PORTC, Enable);
	asm ("nop");
	asm ("nop");
	cbi (PORTC, Enable);
	asm ("nop");
	PORTA=(char)(byte&255);
	cbi (PORTC, SBHE);
	PORTE=MEM_WRITE;
	asm ("nop");
	asm ("nop");
	asm ("nop");
	asm ("nop");
	sbi (PORTC, Enable);
	asm ("nop");
	asm ("nop");
	asm ("nop");
	asm ("nop");
	asm ("nop");
	asm ("nop");
	asm ("nop");
	asm ("nop");
	asm ("nop");
	asm ("nop");
	cbi (PORTC, Enable);
	asm ("nop");
	asm ("nop");
	sbi (PORTC, SBHE);
}

unsigned int io_read16(void)
{unsigned int byte;
	DDRA=0;
	PORTA=255;
	PORTE=IO_READ;
	cbi (PORTC, SBHE);
	asm ("nop");
	sbi (PORTC, Enable);
	asm ("nop");
	asm ("nop");
	asm ("nop");
	asm ("nop");
	asm ("nop");
	asm ("nop");
	asm ("nop");
	asm ("nop");
	asm ("nop");
	asm ("nop");
	byte=PINA;
	cbi (PORTC, Enable);
	asm ("nop");
	asm ("nop");
	PORTE=HIGH_BYTE;
	asm ("nop");
	asm ("nop");
	sbi (PORTC, Enable);
	asm ("nop");
	asm ("nop");
	asm ("nop");
	byte|=PINA*256;
	cbi (PORTC, Enable);
	sbi (PORTC, SBHE);
	return byte;
}

unsigned int mem_read16(void)
{unsigned char byte;
	DDRA=0;
	PORTA=255;
	PORTE=MEM_READ;
	cbi (PORTC, SBHE);
	asm ("nop");
	sbi (PORTC, Enable);
	asm ("nop");
	asm ("nop");
	asm ("nop");
	asm ("nop");
	asm ("nop");
	asm ("nop");
	asm ("nop");
	asm ("nop");
	asm ("nop");
	asm ("nop");
	byte=PINA;
	cbi (PORTC, Enable);
	asm ("nop");
	asm ("nop");
	PORTE=HIGH_BYTE;
	asm ("nop");
	asm ("nop");
	sbi (PORTC, Enable);
	asm ("nop");
	asm ("nop");
	asm ("nop");
	byte|=PINA*256;
	cbi (PORTC, Enable);
	sbi (PORTC, SBHE);
	return byte;
}
